@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (c) 1998-2014 VMware, Inc.  All rights reserved.
+# Copyright (c) 1998-2015 VMware, Inc.  All rights reserved.
 #
 # This script manages the services needed to run VMware software
 
@@ -10,7 +10,7 @@
 # BEGINNING_OF_UTIL_DOT_SH
 #!/bin/sh
 #
-# Copyright (c) 2005-2014 VMware, Inc.  All rights reserved.
+# Copyright (c) 2005-2015 VMware, Inc.  All rights reserved.
 #
 # A few utility functions used by our shell scripts.  Some expect the settings
 # database to already be loaded and evaluated.
@@ -18,6 +18,14 @@
 vmblockmntpt="/proc/fs/vmblock/mountPoint"
 vmblockfusemntpt="/var/run/vmblock-fuse"
 
+vmware_warn_failure() {
+  if [ "`type -t 'echo_warning' 2>/dev/null`" = 'function' ]; then
+    echo_warning
+  else
+    echo -n "$rc_failed"
+  fi
+}
+
 vmware_failed() {
   if [ "`type -t 'echo_failure' 2>/dev/null`" = 'function' ]; then
     echo_failure
@@ -63,6 +71,31 @@ vmware_exec() {
   return 0
 }
 
+
+# Execute a macro, report warning on failure
+vmware_exec_warn() {
+  local msg="$1"  # IN
+  local func="$2" # IN
+  shift 2
+
+  echo -n '   '"$msg"
+
+  if [ "$VMWARE_DEBUG" = 'yes' ]; then
+    (trap '' HUP; "$func" "$@")
+  else
+    (trap '' HUP; "$func" "$@") >/dev/null 2>&1
+  fi
+  if [ "$?" -gt 0 ]; then
+    vmware_warn_failure
+    echo
+    return 1
+  fi
+
+  vmware_success
+  echo
+  return 0
+}
+
 # Execute a macro in the background
 vmware_bg_exec() {
   local msg="$1"  # IN
@@ -366,6 +399,17 @@ vmware_real_modname() {
    fi
 }
 
+vmware_is_upstream() {
+   modname=$1
+   vmware_exec_selinux "$vmdb_answer_LIBDIR/sbin/vmware-modconfig-console \
+                           --install-status" | grep -q "${modname}: other"
+   if [ $? = 0 ]; then
+      echo "yes"
+   else
+      echo 'no'
+   fi
+}
+
 # starts after vmci is loaded
 vmware_start_vsock() {
   real_vmci=$(vmware_real_modname $vmci $vmci_alias)
@@ -393,17 +437,22 @@ vmware_start_vsock() {
 
 # unloads before vmci
 vmware_stop_vsock() {
+  # Nothing to do if module is upstream
+  if [ "`vmware_is_upstream $vsock`" = 'yes' ]; then
+    return 0
+  fi
+
   real_vsock=$(vmware_real_modname $vsock $vsock_alias)
   vmware_unload_module $real_vsock
   rm -f /dev/vsock
 }
 
 is_ESX_running() {
-  if [ ! -f "$vmdb_answer_SBINDIR"/vmware-checkvm ] ; then
+  if [ ! -f "$vmdb_answer_LIBDIR"/sbin/vmware-checkvm ] ; then
     echo no
     return
   fi
-  if "$vmdb_answer_SBINDIR"/vmware-checkvm -p | grep -q ESX; then
+  if "$vmdb_answer_LIBDIR"/sbin/vmware-checkvm -p | grep -q ESX; then
     echo yes
   else
     echo no
@@ -439,8 +488,9 @@ vmware_signal_vmware_user() {
 # A USR1 causes vmware-user to release any references to vmblock or
 # /proc/fs/vmblock/mountPoint, allowing vmblock to unload, but vmware-user
 # to continue running. This preserves the user context vmware-user is
-# running within.
-vmware_unblock_vmware_user() {
+# running within. We also shutdown rpc connections to release usage of
+# vmci/vsocket.
+vmware_user_request_release_resources() {
   vmware_signal_vmware_user 'USR1'
 }
 
@@ -851,25 +901,24 @@ kernel_version_integer() {
 
 # Get the running kernel integer version
 get_version_integer() {
-  local version_uts
-  local v1
-  local v2
-  local v3
+   local IFS
+   local v1
+   local v2
+   local v3
 
-  version_uts=`uname -r`
+   # Split uname -r output, trimming version annotations.
+   #    3.2.0-53-generic -> 3 2 0
+   #    3.10-1 -> 3 10 0
+   #
+   # POSIX shell uses '!' for negation during bracket expansion.
+   # See http://pubs.opengroup.org/onlinepubs/9699919799/utilities/V3_chap02.html.
+   IFS=.
+   set -- `uname -r`
+   v1=${1%%[!0-9]*}; [ -z "$v1" ] && v1=0
+   v2=${2%%[!0-9]*}; [ -z "$v2" ] && v2=0
+   v3=${3%%[!0-9]*}; [ -z "$v3" ] && v3=0
 
-  # There is no double quote around the back-quoted expression on purpose
-  # There is no double quote around $version_uts on purpose
-  set `IFS='.'; echo $version_uts`
-  v1="$1"
-  v2="$2"
-  v3="$3"
-  # There is no double quote around the back-quoted expression on purpose
-  # There is no double quote around $v3 on purpose
-  set `IFS='-ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz'; echo $v3`
-  v3="$1"
-
-  kernel_version_integer "$v1" "$v2" "$v3"
+   kernel_version_integer "$v1" "$v2" "$v3"
 }
 
 #
@@ -932,6 +981,23 @@ vmware_stop_daemon() {
    fi
 }
 
+vmware_start_vgauth()
+{
+  [ -d /var/run/vmware ] || mkdir -p /var/run/vmware
+  vgauthdir=$(dirname $vmdb_answer_LIBDIR)/vmware-vgauth
+  command="$vgauthdir/VGAuthService -b"
+  vmware_exec_selinux "$command"
+}
+
+vmware_stop_vgauth() {
+   local pidfile="/var/run/vmware/vgauthsvclog_pid.txt"
+   vmware_stop_pidfile $pidfile && rm -f $pidfile
+}
+
+vmware_vgauth_enabled() {
+  echo "$vmdb_answer_ENABLE_VGAUTH"
+}
+
 vmware_daemon_status() {
    echo -n "$1 "
    if vmware_check_pidfile "/var/run/$1.pid"; then
@@ -1005,6 +1071,11 @@ vmware_stop_vmci() {
     vmware_stop_vsock
   fi
 
+  # Nothing to do if module is upstream
+  if [ "`vmware_is_upstream $vmci`" = 'yes' ]; then
+    return 0
+  fi
+
   real_vmci=$(vmware_real_modname $vmci $vmci_alias)
   vmware_unload_module $real_vmci
 
@@ -1364,6 +1435,10 @@ main()
 
             if [ "`is_vmtoolsd_needed`" = 'yes' ] ; then
               vmware_exec 'Guest operating system daemon:' vmware_start_daemon $SYSTEM_DAEMON
+            fi
+
+            if [ "`vmware_vgauth_enabled`" = "yes" ]; then
+              vmware_exec 'VGAuthService:' vmware_start_vgauth
               exitcode=$(($exitcode + $?))
             fi
 
@@ -1393,13 +1468,19 @@ main()
               exitcode=$(($exitcode + $?))
             fi
 
+            if [ "`vmware_vgauth_enabled`" = "yes" ]; then
+              vmware_exec 'VGAuthService:' vmware_stop_vgauth
+              exitcode=$(($exitcode + $?))
+            fi
+
+            # Signal vmware-user to release any contact with the blocking fs, closing rpc connections etc.
+            vmware_exec 'VMware User Agent (vmware-user):' vmware_user_request_release_resources
+            rv=$?
+            exitcode=$(($exitcode + $rv))
+
             if [ "`is_vmblock_needed`" = 'yes' ] ; then
-            # Signal vmware-user to release any contact with the blocking fs.
-               vmware_exec 'VMware User Agent (vmware-user):' vmware_unblock_vmware_user
-               rv=$?
-               exitcode=$(($exitcode + $rv))
-           # If unblocking vmware-user fails then stopping and unloading vmblock
-           # probably will also fail.
+               # If unblocking vmware-user fails then stopping and unloading vmblock
+               # probably will also fail.
                if [ $rv -eq 0 ]; then
                   vmware_exec 'Blocking file system:' vmware_stop_vmblock
                   exitcode=$(($exitcode + $?))
@@ -1421,12 +1502,12 @@ main()
 
          # vsock requires vmci to work so it must be unloaded before vmci
             if [ "`is_vsock_needed`" = 'yes' ]; then
-               vmware_exec 'VM communication interface socket family:' vmware_stop_vsock
+               vmware_exec_warn 'VM communication interface socket family:' vmware_stop_vsock
                exitcode=$(($exitcode + $?))
             fi
 
             if [ "`is_vmci_needed`" = 'yes' ]; then
-               vmware_exec 'VM communication interface:' vmware_stop_vmci
+               vmware_exec_warn 'VM communication interface:' vmware_stop_vmci
                exitcode=$(($exitcode + $?))
             fi
 
