@@ -1,11 +1,11 @@
 /* **********************************************************
- * Copyright (c) 2007-2014 VMware, Inc.  All rights reserved.
+ * Copyright (c) 2007-2015 VMware, Inc.  All rights reserved.
  * **********************************************************/
 
 /*
  * vmci_sockets.h --
  *
- *    VMCI sockets public constants and types.
+ *    vSockets public constants and types.
  */
 
 #ifndef _VMCI_SOCKETS_H_
@@ -545,9 +545,9 @@ struct uuid_2_cid {
       int fd;
       unsigned int version;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return VMCI_SOCKETS_INVALID_VERSION;
          }
@@ -567,9 +567,9 @@ struct uuid_2_cid {
     *
     * \brief Retrieve the address family value for vSockets.
     *
-    * Returns the value to be used for the VMCI Sockets address family.
+    * Returns the value to be used for the vSockets address family.
     * This value should be used as the domain argument to \c socket(2) (when
-    * you might otherwise use \c AF_INET).  For VMCI Socket-specific options,
+    * you might otherwise use \c AF_INET).  For vSocket-specific options,
     * this value should also be used for the level argument to
     * \c setsockopt(2) (when you might otherwise use \c SOL_TCP).
     *
@@ -605,11 +605,34 @@ struct uuid_2_cid {
    static inline int VMCISock_GetAFValueFd(int *outFd)
    {
       int fd;
-      int family;
+      int family = -1;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+#if defined(linux)
+      /*
+       * vSockets is now in mainline kernel with address family 40.  As part
+       * of upstreaming, we removed the IOCTL we use below to determine the
+       * address family.  So to handle both a new and old kernel we do this:
+       * 1. Check if our family already exists by making a socket with it.
+       *    Some weird kernel might claim this too, but it's very unlikely
+       *    (Linus' tree has us at 40, and that's what we care about).
+       * 2. If that fails, try the normal IOCTL path, since it's probably an
+       *    older kernel with vSockets from Tools.
+       * 3. If that fails, then vSockets really isn't available.
+       */
+#define AF_VSOCK_LOCAL 40
+      {
+         int s = socket(AF_VSOCK_LOCAL, SOCK_DGRAM, 0);
+         if (s != -1) {
+            close(s);
+            return AF_VSOCK_LOCAL;
+         }
+      }
+#undef AF_VSOCK_LOCAL
+#endif // linux
+
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return -1;
          }
@@ -635,9 +658,9 @@ struct uuid_2_cid {
     *
     * \brief Retrieve the address family value for vSockets.
     *
-    * Returns the value to be used for the VMCI Sockets address family.
+    * Returns the value to be used for the vSockets address family.
     * This value should be used as the domain argument to \c socket(2) (when
-    * you might otherwise use \c AF_INET).  For VMCI Socket-specific options,
+    * you might otherwise use \c AF_INET).  For vSocket-specific options,
     * this value should also be used for the level argument to
     * \c setsockopt(2) (when you might otherwise use \c SOL_TCP).
     *
@@ -728,9 +751,9 @@ struct uuid_2_cid {
       int fd;
       unsigned int contextId;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return VMADDR_CID_ANY;
          }
@@ -781,9 +804,9 @@ struct uuid_2_cid {
       int fd;
       struct uuid_2_cid io;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return VMADDR_CID_ANY;
          }
