@@ -416,7 +416,11 @@ int vmxnet3_set_features(struct net_device *netdev, compat_netdev_features_t fea
 	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
 	unsigned long flags;
 	compat_netdev_features_t changed = features ^ netdev->features;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)
+	if (changed & (NETIF_F_RXCSUM | NETIF_F_LRO | NETIF_F_HW_VLAN_CTAG_RX)) {
+#else
 	if (changed & (NETIF_F_RXCSUM | NETIF_F_LRO | NETIF_F_HW_VLAN_RX)) {
+#endif
 		if (features & NETIF_F_RXCSUM) {
 			adapter->shared->devRead.misc.uptFeatures |=
 				UPT1_F_RXCSUM;
@@ -437,7 +441,11 @@ int vmxnet3_set_features(struct net_device *netdev, compat_netdev_features_t fea
 			adapter->lro = FALSE;
 		}
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)
+		if (features & NETIF_F_HW_VLAN_CTAG_RX)
+#else
 		if (features & NETIF_F_HW_VLAN_RX)
+#endif
 			adapter->shared->devRead.misc.uptFeatures |=
 				UPT1_F_RXVLAN;
 		else
@@ -644,12 +652,12 @@ vmxnet3_get_ringparam(struct net_device *netdev,
 	param->rx_max_pending = VMXNET3_RX_RING_MAX_SIZE;
 	param->tx_max_pending = VMXNET3_TX_RING_MAX_SIZE;
 	param->rx_mini_max_pending = 0;
-	param->rx_jumbo_max_pending = 0;
+	param->rx_jumbo_max_pending = VMXNET3_RX_RING2_MAX_SIZE;
 
 	param->rx_pending = adapter->rx_queue[0].rx_ring[0].size;
 	param->tx_pending = adapter->tx_queue[0].tx_ring.size;
 	param->rx_mini_pending = 0;
-	param->rx_jumbo_pending = 0;
+	param->rx_jumbo_pending = adapter->rx_queue[0].rx_ring[1].size;
 }
 
 
@@ -658,7 +666,7 @@ vmxnet3_set_ringparam(struct net_device *netdev,
 		      struct ethtool_ringparam *param)
 {
 	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
-	u32 new_tx_ring_size, new_rx_ring_size;
+	u32 new_tx_ring_size, new_rx_ring_size, new_rx_ring2_size;
 	u32 sz;
 
 	if (param->tx_pending == 0 ||
@@ -669,6 +677,10 @@ vmxnet3_set_ringparam(struct net_device *netdev,
 	    param->rx_pending >	VMXNET3_RX_RING_MAX_SIZE)
 		return -EINVAL;
 
+	if (param->rx_jumbo_pending == 0 ||
+            param->rx_jumbo_pending > VMXNET3_RX_RING2_MAX_SIZE)
+                return -EINVAL;
+
 	/* if adapter not yet initialized, do nothing */
 	if (adapter->rx_buf_per_pkt == 0) {
 		printk(KERN_INFO "%s: adapter not completely initialized, "
@@ -694,13 +706,22 @@ vmxnet3_set_ringparam(struct net_device *netdev,
 	BUG_ON(new_rx_ring_size > VMXNET3_RX_RING_MAX_SIZE);
 	BUG_ON(new_rx_ring_size % sz != 0);
 
+	/* ring1 has to be a multiple of VMXNET3_RING_SIZE_ALIGN */
+        new_rx_ring2_size = (param->rx_jumbo_pending + VMXNET3_RING_SIZE_MASK) &
+                ~VMXNET3_RING_SIZE_MASK;
+        new_rx_ring2_size = min_t(u32, new_rx_ring2_size,
+                                  VMXNET3_RX_RING2_MAX_SIZE);
+        BUG_ON(new_rx_ring2_size > VMXNET3_RX_RING2_MAX_SIZE);
+        BUG_ON(new_rx_ring2_size % VMXNET3_RING_SIZE_ALIGN != 0);
+
 	if(adapter->use_adaptive_ring) {
 		adapter->use_adaptive_ring = FALSE;
 		printk(KERN_INFO "%s: User specified custom ring size, "
 		       "disabling adaptive ring support.\n", netdev->name);
 	}
 
-	return vmxnet3_set_ringsize(netdev, new_tx_ring_size, new_rx_ring_size);
+	return vmxnet3_set_ringsize(netdev, new_tx_ring_size, new_rx_ring_size,
+                                    new_rx_ring2_size);
 }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
